/* Winsock startup and more.
   Copyright (C) 1995,96,97  Darko Budor

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

#ifdef WINDOWS

#include <stdio.h>
#include <stdlib.h>
#include <winsock.h>
#include <string.h>
#include <assert.h>

#include <config.h>
#include "wget.h"
#include "options.h"
#include "utils.h"
#include "url.h"

extern struct options opt;
extern char *argv0;

void ws_cleanup(void);
BOOL WINAPI ws_handler(DWORD);
void ws_hangup(void);

static char *title_buf = NULL;

static int isNT = 0;

void ws_startup()
{
    WORD requested;
    WSADATA data;
	int err;
    OSVERSIONINFO os;

    if (GetVersionEx(&os) == TRUE
            && os.dwPlatformId == VER_PLATFORM_WIN32_WINDOWS)
       isNT = 1;

    requested = MAKEWORD(1, 1);
    err = WSAStartup(requested, &data);

    if (err != 0) {
		fprintf(stderr, "Couldn't find usable socket driver.\n");
		exit(1);
	}

    if (LOBYTE(requested) < 1 || (LOBYTE(requested) == 1 &&
     HIBYTE(requested) < 1))
	{
		fprintf(stderr, "Couldn't find usable socket driver.\n");
		WSACleanup();
		exit(1);
	}
	atexit(ws_cleanup);
	SetConsoleCtrlHandler(ws_handler, TRUE);
}

void ws_cleanup() {
	WSACleanup();
}

void ws_backgnd()
{
   ws_hangup();
   if (!isNT)
      FreeConsole();
}

BOOL WINAPI ws_handler(DWORD dwEvent)
{
    char *buf;
	switch (dwEvent)
	{
		
#ifdef CTRLC_BACKGND
          case CTRL_C_EVENT:
#endif
#ifdef CTRLBREAK_BACKGND
          case CTRL_BREAK_EVENT:
#endif
            ws_backgnd();
			break;
		case CTRL_SHUTDOWN_EVENT:
		case CTRL_CLOSE_EVENT:
		case CTRL_LOGOFF_EVENT:
		default:
			WSACleanup();
			return (FALSE);
	}
    return TRUE;
}

void ws_hangup()
{
   char *logname;
   int count;

   if (opt.lfile != stdout)
      return;
   for (count = 0; 
      (logname = unique_name(DEFAULT_LOGFILE, count)) == NULL;
      count++);
   opt.lfile = fopen(logname, "w");
   if (opt.lfile == NULL)
   {
      printf("%s: %s\n", logname, mystrerror(errno));
      opt.lfile = fopen("NUL", "w");
      assert(opt.lfile != NULL);
   }
   /* Change the buffering. */
   setbuf(opt.lfile, NULL);
   printf("\nCTRL+Break received, redirecting output to %s.\n", logname);
   printf("Execution continued in background.\n");
   printf("You may stop Wget by pressing CTRL+ALT+DELETE.\n");
   free(logname);
}

void ws_changetitle(char *url, int nurl)
{
   int len = 0;
   if (!nurl)
      return;
   title_buf = (char *)nmalloc(strlen(url) + 20);
   sprintf(title_buf, "Wget %s%s", url, nurl == 1 ? "" : " ...");
   SetConsoleTitle(title_buf);
}

char *read_registry(HKEY hkey, char *subkey, char *valuename, char *buf, int *len)
{
   HKEY result;
   DWORD size = *len;
   DWORD type = REG_SZ;
   if (RegOpenKeyEx(hkey, subkey, NULL, KEY_READ, &result) != ERROR_SUCCESS)
      return NULL;
   if (RegQueryValueEx(result, valuename, NULL, &type, buf, &size) != ERROR_SUCCESS)
      buf = NULL;
   *len = size;
   RegCloseKey(result);
   return buf;
}

char *ws_mypath()
{
   char *buffer;
   int rrr;
   char *ptr = strrchr(argv0, '\\');
   if (ptr)
   {
      *(ptr + 1) = '\0';
      return argv0;
   }
   buffer = (char*) nmalloc(256);
   if (!!(rrr = SearchPath(NULL, argv0, strchr(argv0, '.') ? NULL : ".EXE",
               256, buffer, &ptr))
      && rrr <= 256)
   {
      *ptr = '\0';
      return buffer;
   }
   free(buffer);
   return NULL;
}


void *ws_help(char *name)
{
   char *buf;
   char *mypath = ws_mypath();
   if (mypath) {
      buf = (char *)nmalloc(strlen(mypath) + 1 + strlen(name) + 3 + 1);
      sprintf(buf, "%s%s.HLP", mypath, name);
#ifdef DEBUG
      printf("Starting WinHelp %s\n", buf);
#endif
      WinHelp(NULL, buf, HELP_INDEX, NULL);
   }
}

#endif /* WINDOWS */
